@@ -1,10 +1,7 @@
 #!/usr/bin/env python
-# -*- coding: ascii -*-
-# <p>Copyright (c) 2005-2012 Stephen John Machin, Lingfo Pty Ltd</p>
-# <p>This script is part of the xlrd package, which is released under a
-# BSD-style licence.</p>
-
-from __future__ import print_function
+# Copyright (c) 2005-2012 Stephen John Machin, Lingfo Pty Ltd
+# This script is part of the xlrd package, which is released under a
+# BSD-style licence.
 
 cmd_doc = """
 Commands:
@@ -32,14 +29,15 @@ xfc             Print "XF counts" and cell-type counts -- see code for details
 
 options = None
 if __name__ == "__main__":
+    import xlrd3 as xlrd
+    import sys
+    import time
+    import glob
+    import traceback
+    import gc
 
-    PSYCO = 0
+    from xlrd.timemachine import xrange, REPR
 
-    import xlrd
-    import sys, time, glob, traceback, pprint, gc
-    
-    from xlrd.timemachine import xrange
-    
 
     class LogHandler(object):
 
@@ -47,17 +45,17 @@ if __name__ == "__main__":
             self.logfileobj = logfileobj
             self.fileheading = None
             self.shown = 0
-            
+
         def setfileheading(self, fileheading):
             self.fileheading = fileheading
             self.shown = 0
-            
+
         def write(self, text):
             if self.fileheading and not self.shown:
                 self.logfileobj.write(self.fileheading)
                 self.shown = 1
             self.logfileobj.write(text)
-        
+
     null_cell = xlrd.empty_cell
 
     def show_row(bk, sh, rowx, colrange, printit):
@@ -90,9 +88,8 @@ if __name__ == "__main__":
             if cty == xlrd.XL_CELL_DATE:
                 try:
                     showval = xlrd.xldate_as_tuple(cval, dmode)
-                except xlrd.XLDateError:
-                    e1, e2 = sys.exc_info()[:2]
-                    showval = "%s:%s" % (e1.__name__, e2)
+                except xlrd.XLDateError as e:
+                    showval = "%s:%s" % (type(e).__name__, e)
                     cty = xlrd.XL_CELL_ERROR
             elif cty == xlrd.XL_CELL_ERROR:
                 showval = xlrd.error_text_from_code.get(cval, '<Unknown error code 0x%02x>' % cval)
@@ -109,13 +106,13 @@ if __name__ == "__main__":
             % (bk.codepage, bk.encoding, bk.countries))
         print("Last saved by: %r" % bk.user_name)
         print("Number of data sheets: %d" % bk.nsheets)
-        print("Pickleable: %d; Use mmap: %d; Formatting: %d; On demand: %d"
-            % (bk.pickleable, bk.use_mmap, bk.formatting_info, bk.on_demand))
+        print("Use mmap: %d; Formatting: %d; On demand: %d"
+            % (bk.use_mmap, bk.formatting_info, bk.on_demand))
         print("Ragged rows: %d" % bk.ragged_rows)
         if bk.formatting_info:
             print("FORMATs: %d, FONTs: %d, XFs: %d"
                 % (len(bk.format_list), len(bk.font_list), len(bk.xf_list)))
-        if not options.suppress_timing:        
+        if not options.suppress_timing:
             print("Load time: %.2f seconds (stage 1) %.2f seconds (stage 2)"
                 % (bk.load_time_stage_1, bk.load_time_stage_2))
         print()
@@ -189,16 +186,16 @@ if __name__ == "__main__":
             nrows, ncols = sh.nrows, sh.ncols
             colrange = range(ncols)
             anshow = min(nshow, nrows)
-            print("sheet %d: name = %r; nrows = %d; ncols = %d" %
-                (shx, sh.name, sh.nrows, sh.ncols))
+            print("sheet %d: name = %s; nrows = %d; ncols = %d" %
+                (shx, REPR(sh.name), sh.nrows, sh.ncols))
             if nrows and ncols:
                 # Beat the bounds
                 for rowx in xrange(nrows):
                     nc = sh.row_len(rowx)
                     if nc:
-                        _junk = sh.row_types(rowx)[nc-1]
-                        _junk = sh.row_values(rowx)[nc-1]
-                        _junk = sh.cell(rowx, nc-1)
+                        sh.row_types(rowx)[nc-1]
+                        sh.row_values(rowx)[nc-1]
+                        sh.cell(rowx, nc-1)
             for rowx in xrange(anshow-1):
                 if not printit and rowx % 10000 == 1 and rowx > 1:
                     print("done %d rows" % (rowx-1,))
@@ -212,7 +209,7 @@ if __name__ == "__main__":
         bk_header(bk)
         for shx in range(bk.nsheets):
             sh = bk.sheet_by_index(shx)
-            nrows, ncols = sh.nrows, sh.ncols
+            nrows = sh.nrows
             print("sheet %d: name = %r; nrows = %d; ncols = %d" %
                 (shx, sh.name, sh.nrows, sh.ncols))
             # Access all xfindexes to force gathering stats
@@ -230,7 +227,7 @@ if __name__ == "__main__":
 
     def main(cmd_args):
         import optparse
-        global options, PSYCO
+        global options
         usage = "\n%prog [options] command [input-file-patterns]\n" + cmd_doc
         oparser = optparse.OptionParser(usage)
         oparser.add_option(
@@ -241,10 +238,6 @@ if __name__ == "__main__":
             "-v", "--verbosity",
             type="int", default=0,
             help="level of information and diagnostics provided")
-        oparser.add_option(
-            "-p", "--pickleable",
-            type="int", default=1,
-            help="1: ensure Book object is pickleable (default); 0: don't bother")
         oparser.add_option(
             "-m", "--mmap",
             type="int", default=-1,
@@ -257,8 +250,8 @@ if __name__ == "__main__":
             "-f", "--formatting",
             type="int", default=0,
             help="0 (default): no fmt info\n"
-                 "1: fmt info (all cells)\n"
-            )
+                 "1: fmt info (all cells)\n",
+        )
         oparser.add_option(
             "-g", "--gc",
             type="int", default=0,
@@ -323,34 +316,29 @@ if __name__ == "__main__":
                     n_unreachable = gc.collect()
                     if n_unreachable:
                         print("GC before open:", n_unreachable, "unreachable objects")
-                if PSYCO:
-                    import psyco
-                    psyco.full()
-                    PSYCO = 0
                 try:
                     t0 = time.time()
-                    bk = xlrd.open_workbook(fname,
+                    bk = xlrd.open_workbook(
+                        fname,
                         verbosity=options.verbosity, logfile=logfile,
-                        pickleable=options.pickleable, use_mmap=mmap_arg,
+                        use_mmap=mmap_arg,
                         encoding_override=options.encoding,
                         formatting_info=fmt_opt,
                         on_demand=options.on_demand,
                         ragged_rows=options.ragged_rows,
-                        )
+                    )
                     t1 = time.time()
                     if not options.suppress_timing:
                         print("Open took %.2f seconds" % (t1-t0,))
-                except xlrd.XLRDError:
-                    e0, e1 = sys.exc_info()[:2]
-                    print("*** Open failed: %s: %s" % (e0.__name__, e1))
+                except xlrd.XLRDError as e:
+                    print("*** Open failed: %s: %s" % (type(e).__name__, e))
                     continue
                 except KeyboardInterrupt:
                     print("*** KeyboardInterrupt ***")
                     traceback.print_exc(file=sys.stdout)
                     sys.exit(1)
-                except:
-                    e0, e1 = sys.exc_info()[:2]
-                    print("*** Open failed: %s: %s" % (e0.__name__, e1))
+                except BaseException as e:
+                    print("*** Open failed: %s: %s" % (type(e).__name__, e))
                     traceback.print_exc(file=sys.stdout)
                     continue
                 t0 = time.time()
@@ -396,7 +384,8 @@ if __name__ == "__main__":
         main(av)
     firstarg = av[0].lower()
     if firstarg == "hotshot":
-        import hotshot, hotshot.stats
+        import hotshot
+        import hotshot.stats
         av = av[1:]
         prof_log_name = "XXXX.prof"
         prof = hotshot.Profile(prof_log_name)
@@ -415,8 +404,5 @@ if __name__ == "__main__":
         import pstats
         p = pstats.Stats('YYYY.prof')
         p.strip_dirs().sort_stats('cumulative').print_stats(30)
-    elif firstarg == "psyco":
-        PSYCO = 1
-        main(av[1:])
     else:
         main(av)
